@@ -26,37 +26,47 @@ static unsigned short unpack16(unsigned char *s)
     return ntohs(ashort);
 }
 
-#define WRITE_BUFF(buf, size, r) \
+#define WRITE_BUFF(buf, size, r)                                \
     if (ap_rwrite(buf, size, r) < 0) { return APR_EGENERAL; }
 
 static int scoreboard_send(request_rec *r)
 {
-    int i, psize, ssize, tsize;
-    char buf[SIZE16*2];
+    int psize, ssize, tsize;
+    char buf[SIZE16*4];
     char *ptr = buf;
+    int server_limit, thread_limit;
 
-    for (i = 0; i < server_limit; i++) {
-        if (!ap_scoreboard_image->parent[i].pid) {
-            break;
-        }
-    }
-    
-    psize = i * sizeof(process_score);
-    ssize = i * sizeof(worker_score);
+    ap_mpm_query(AP_MPMQ_HARD_LIMIT_THREADS, &thread_limit);
+    ap_mpm_query(AP_MPMQ_HARD_LIMIT_DAEMONS, &server_limit);
+
+    psize = sizeof(process_score) * server_limit;
+    ssize = sizeof(worker_score)  * server_limit * thread_limit;
     tsize = psize + ssize + sizeof(global_score) + sizeof(buf);
 
     pack16(ptr, psize);
     ptr += SIZE16;
     pack16(ptr, ssize);
+    ptr += SIZE16;
+    pack16(ptr, server_limit);
+    ptr += SIZE16;
+    pack16(ptr, thread_limit);
+
+#if 0
+    ap_log_error(APLOG_MARK, APLOG_ERR, 0, modperl_global_get_server_rec(),
+                 "send: sizes server_limit=%d, thread_num=%d, psize=%d, "
+                 "ssize=%d, %d, %d, %d\n",
+                 server_limit, thread_limit, psize, ssize,
+                 sizeof(global_score), sizeof(buf), tsize);
+#endif
 
     ap_set_content_length(r, tsize);
     r->content_type = REMOTE_SCOREBOARD_TYPE;
     
     if (!r->header_only) {
-	WRITE_BUFF(&buf[0],                          sizeof(buf),          r);
-	WRITE_BUFF(&ap_scoreboard_image->parent[0],  psize,                r);
-	WRITE_BUFF(&ap_scoreboard_image->servers[0], ssize,                r);
-	WRITE_BUFF(&ap_scoreboard_image->global,     sizeof(global_score), r);
+        WRITE_BUFF(&buf[0],                         sizeof(buf),          r);
+        WRITE_BUFF(&ap_scoreboard_image->parent[0], psize,                r);
+        WRITE_BUFF(ap_scoreboard_image->servers[0], ssize,                r);
+        WRITE_BUFF(ap_scoreboard_image->global,     sizeof(global_score), r);
     }
 
     return APR_SUCCESS;
